@@ -23,7 +23,7 @@ OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 ********************************************************/
-/* $XFree86: xc/lib/X11/XKBExtDev.c,v 3.6 2006/01/09 14:58:37 dawes Exp $ */
+/* $XFree86: xc/lib/X11/XKBExtDev.c,v 3.7 2007/04/09 15:37:11 tsi Exp $ */
 
 #include <stdio.h>
 #define NEED_REPLIES
@@ -794,6 +794,7 @@ XkbSetDeviceButtonActions(	Display *		dpy,
     changes.leds.led_id=	XkbXINone;
     changes.leds.defined=	0;
     size= nLeds= 		0;
+    _InitLedStuff(&lstuff,changes.changed,devi);
     if (_XkbSetDeviceInfoSize(devi,&changes,NULL,&size,&nLeds)!=Success)
 	return False;
     LockDisplay(dpy);
@@ -815,5 +816,6 @@ XkbSetDeviceButtonActions(	Display *		dpy,
     }
     UnlockDisplay(dpy);
     SyncHandle();
+    _FreeLedStuff(&lstuff);
     return ok;
 }
