@@ -1,18 +1,16 @@
-/* $Xorg: Initialize.c,v 1.8 2001/02/09 02:03:55 xorgcvs Exp $ */
-
 /***********************************************************
 Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts
 Copyright 1993 by Sun Microsystems, Inc. Mountain View, CA.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Digital or Sun not be
 used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.  
+software without specific, written prior permission.
 
 DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
@@ -32,7 +30,7 @@ OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
-/* $XFree86: xc/lib/Xt/Initialize.c,v 3.21 2003/04/21 16:34:27 herrb Exp $ */
+/* $XFree86: xc/lib/Xt/Initialize.c,v 3.25 2006/06/19 13:43:24 tsi Exp $ */
 
 /*
 
@@ -110,7 +108,7 @@ in this Software without prior written authorization from The Open Group.
 /*
  This is a set of default records describing the command line arguments that
  Xlib will parse and set into the resource data base.
- 
+
  This list is applied before the users list to enforce these defaults.  This is
  policy, which the toolkit avoids but I hate differing programs at this level.
 */
@@ -156,27 +154,27 @@ static void GetHostname (
     int len;
     struct utsname name;
 
-    if (maxlen <= 0 || buf == NULL)
+    if ((maxlen <= 0) || (buf == NULL) || (uname(&name) < 0))
 	return;
 
-    uname (&name);
-    len = strlen (name.nodename);
-    if (len >= maxlen) len = maxlen;
-    (void) strncpy (buf, name.nodename, len-1);
-    buf[len-1] = '\0';
+    len = strlen(name.nodename);
+    if (len >= maxlen)
+	len = maxlen - 1;
+    (void) strncpy(buf, name.nodename, len);
+    buf[len] = '\0';
 #else
     if (maxlen <= 0 || buf == NULL)
 	return;
 
     buf[0] = '\0';
-    (void) gethostname (buf, maxlen);
-    buf [maxlen - 1] = '\0';
+    (void) gethostname(buf, maxlen);
+    buf[maxlen - 1] = '\0';
 #endif
 }
 
 
 #ifdef SUNSHLIB
-void _XtInherit()
+void _XtInherit(void)
 {
     extern void __XtInherit();
     __XtInherit();
@@ -184,7 +182,67 @@ void _XtInherit()
 #define _XtInherit __XtInherit
 #endif
 
-void _XtInherit()
+
+#if defined(__CYGWIN__)
+/*
+ * The Symbol _XtInherit is used in two different manners.
+ * First it could be used as a generic function and second
+ * as an absolute address reference, which will be used to
+ * check the initialisation process of several other libraries.
+ * Because of this the symbol must be accessable by all
+ * client dll's and applications.  In unix environments
+ * this is no problem, because the used shared libraries
+ * format (elf) supports this immediatly.  Under Windows
+ * this isn't true, because a functions address in a dll
+ * is different from the same function in another dll or
+ * applications, because the used Portable Executable
+ * File adds a code stub to each client to provide the
+ * exported symbol name.  This stub uses an indirect
+ * pointer to get the original symbol address, which is
+ * then jumped to, like in this example:
+ *
+ * --- client ---                                     --- dll ----
+ *  ...
+ *  call foo
+ *
+ * foo: jmp (*_imp_foo)               ---->           foo: ....
+ *      nop
+ *      nop
+ *
+ * _imp_foo: .long <index of foo in dll export table, is
+ *		    set to the real address by the runtime linker>
+ *
+ * Now it is clear why the clients symbol foo isn't the same
+ * as in the dll and we can think about how to deal which
+ * this two above mentioned requirements, to export this
+ * symbol to all clients and to allow calling this symbol
+ * as a function.  The solution I've used exports the
+ * symbol _XtInherit as data symbol, because global data
+ * symbols are exported to all clients.  But how to deal
+ * with the second requirement, that this symbol should
+ * be used as function.  The Trick is to build a little
+ * code stub in the data section in the exact manner as
+ * above explained.  This is done with the assembler code
+ * below.
+ *
+ * Ralf Habacker
+ *
+ * References:
+ * msdn          http://msdn.microsoft.com/msdnmag/issues/02/02/PE/PE.asp
+ * cygwin-xfree: http://www.cygwin.com/ml/cygwin-xfree/2003-10/msg00000.html
+ */
+
+asm (".data\n\
+ .globl __XtInherit        \n\
+ __XtInherit:      jmp *_y \n\
+  _y: .long ___XtInherit   \n\
+    .text                 \n");
+
+#define _XtInherit __XtInherit
+#endif
+
+
+void _XtInherit(void)
 {
     XtErrorMsg("invalidProcedure","inheritanceProc",XtCXtToolkitError,
             "Unresolved inheritance operation",
@@ -192,7 +250,7 @@ void _XtInherit()
 }
 
 
-void XtToolkitInitialize()
+void XtToolkitInitialize(void)
 {
     static Boolean initialized = False;
 
@@ -215,14 +273,14 @@ void XtToolkitInitialize()
     /* Some apps rely on old (broken) XtAppPeekEvent behavior */
     if(getenv("XTAPPPEEKEVENT_SKIPTIMER"))
 	XtAppPeekEvent_SkipTimer = True;
-    else 
+    else
 	XtAppPeekEvent_SkipTimer = False;
 }
 
 
-String _XtGetUserName(dest, len)
-    String dest;
-    int len;
+String _XtGetUserName(
+    String dest,
+    int len)
 {
 #ifdef WIN32
     String ptr = NULL;
@@ -260,7 +318,7 @@ static String GetRootDirName(
 {
 #ifdef WIN32
     register char *ptr1;
-    register char *ptr2;
+    register char *ptr2 = NULL;
     int len1 = 0, len2 = 0;
 
     if (ptr1 = getenv("HOME")) {	/* old, deprecated */
@@ -355,7 +413,7 @@ static void CombineUserDefaults(
 	XrmCombineDatabase(XrmGetStringDatabase(dpy_defaults), pdb, False);
     } else {
 	char filename[PATH_MAX];
-	(void) GetRootDirName(filename, 
+	(void) GetRootDirName(filename,
 			PATH_MAX - strlen (slashDotXdefaults) - 1);
 	(void) strcat(filename, slashDotXdefaults);
 	(void)XrmCombineFileDatabase(filename, pdb, False);
@@ -425,7 +483,7 @@ XtLanguageProc XtSetLanguageProc(
         app->langProcRec.closure = closure;
 	UNLOCK_PROCESS;
 	UNLOCK_APP(app);
-    } else {    
+    } else {
 	/* set langProcRec for all application contexts */
         ProcessContext process;
 
@@ -445,8 +503,8 @@ XtLanguageProc XtSetLanguageProc(
     return (old ? old : _XtDefaultLanguageProc);
 }
 
-XrmDatabase XtScreenDatabase(screen)
-    Screen *screen;
+XrmDatabase XtScreenDatabase(
+    Screen *screen)
 {
     int scrno;
     Bool doing_def;
@@ -488,11 +546,11 @@ XrmDatabase XtScreenDatabase(screen)
 	    int len;
 	    char *slashDotXdefaultsDash = "/.Xdefaults-";
 
-	    (void) GetRootDirName(filename = filenamebuf, 
+	    (void) GetRootDirName(filename = filenamebuf,
 			PATH_MAX - strlen (slashDotXdefaultsDash) - 1);
 	    (void) strcat(filename, slashDotXdefaultsDash);
 	    len = strlen(filename);
-	    GetHostname (filename+len, PATH_MAX-len);
+	    GetHostname (filename+len, PATH_MAX-1-len);
 	}
 	(void)XrmCombineFileDatabase(filename, &db, False);
     }
@@ -563,9 +621,11 @@ static void _MergeOptionTables(
     Cardinal *num_dst)
 {
     XrmOptionDescRec *table, *endP;
-    register XrmOptionDescRec *opt1, *whereP, *dstP; 
-    const register XrmOptionDescRec *opt2;
-    int i1, i2, dst_len, order;
+    register XrmOptionDescRec *opt1, *whereP, *dstP;
+    register const XrmOptionDescRec *opt2;
+    int i1;
+    Cardinal i2;
+    int dst_len, order;
     Boolean found;
     enum {Check, NotSorted, IsSorted} sort_order = Check;
 
@@ -663,13 +723,15 @@ static Boolean _GetResource(
     return False;
 }
 
-XrmDatabase _XtPreparseCommandLine(urlist, num_urs, argc, argv, applName,
-				   displayName, language)
-    XrmOptionDescRec *urlist;
-    Cardinal num_urs;
-    int argc;
-    String *argv;
-    String *applName, *displayName, *language;	/* return */
+XrmDatabase _XtPreparseCommandLine(
+    XrmOptionDescRec *urlist,
+    Cardinal num_urs,
+    int argc,
+    String *argv,
+    /* return */
+    String *applName,
+    String *displayName,
+    String *language)
 {
     XrmDatabase db = 0;
     XrmOptionDescRec *options;
@@ -713,7 +775,7 @@ XrmDatabase _XtPreparseCommandLine(urlist, num_urs, argc, argv, applName,
     return db;
 }
 
-  
+
 static void GetLanguage(
     Display *dpy,
     XtPerDisplay pd)
@@ -892,7 +954,7 @@ XtAppSetFallbackResources(
     UNLOCK_APP(app_context);
 }
 
-	
+
 Widget XtOpenApplication(XtAppContext *app_context_return,
 			 _Xconst char *application_class,
 			 XrmOptionDescRec *options, Cardinal num_options,
@@ -906,9 +968,9 @@ Widget XtOpenApplication(XtAppContext *app_context_return,
     Widget root;
     Arg args[3], *merged_args;
     Cardinal num = 0;
-    
+
     XtToolkitInitialize(); /* cannot be moved into _XtAppInit */
-    
+
     dpy = _XtAppInit(&app_con, (String)application_class, options, num_options,
 		     argc_in_out, &argv_in_out, fallback_resources);
 
@@ -922,7 +984,7 @@ Widget XtOpenApplication(XtAppContext *app_context_return,
 
     root = XtAppCreateShell(NULL, application_class, widget_class, dpy,
 			    merged_args, num);
-    
+
     if (app_context_return)
 	*app_context_return = app_con;
 
@@ -932,7 +994,7 @@ Widget XtOpenApplication(XtAppContext *app_context_return,
     return root;
 }
 
-	
+
 Widget
 XtAppInitialize(
     XtAppContext * app_context_return,
@@ -946,7 +1008,7 @@ XtAppInitialize(
     Cardinal num_args_in)
 {
     return XtOpenApplication(app_context_return, application_class,
-			     options, num_options, 
+			     options, num_options,
 			     argc_in_out, argv_in_out, fallback_resources,
 			     applicationShellWidgetClass,
 			     args_in, num_args_in);
@@ -954,7 +1016,7 @@ XtAppInitialize(
 
 
 /*ARGSUSED*/
-Widget 
+Widget
 XtInitialize(
     _Xconst char* name,
     _Xconst char* classname,
@@ -972,6 +1034,6 @@ XtInitialize(
 
     LOCK_PROCESS;
     process->defaultAppContext = app_con;
-    UNLOCK_PROCESS; 
+    UNLOCK_PROCESS;
     return root;
 }
