@@ -1,8 +1,10 @@
-/* $XFree86: xc/programs/Xserver/dbe/dbeproc.h,v 1.2 2006/02/19 15:51:18 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/dbe/dbeproc.h,v 1.3 2007/01/04 02:48:12 tsi Exp $ */
 
 #ifndef DBEPROC_H
 #define DBEPROC_H 1
 
+#include "extnsionst.h"
+
 #ifndef DBE_EXT_INIT_ONLY
 
 #include "dbestruct.h"
@@ -16,6 +18,6 @@ void DbeRegisterFunction(ScreenPtr pScreen, DbeInitFuncPtr funct);
 
 #endif /* DBE_EXT_INIT_ONLY */
 
-void DbeExtensionInit(void);
+void DbeExtensionInit(INITARGS);
 
 #endif /* DBEPROC_H */
