@@ -15,7 +15,7 @@
  *
  * Author:  Doug Moran, SRI
  */
-/* $XFree86: xc/programs/Xserver/hw/sun/constype.c,v 3.11 2006/01/09 14:59:49 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/sun/constype.c,v 3.12 2006/12/27 04:20:25 tsi Exp $ */
 
 /*
 SUN-SPOTS DIGEST         Thursday, 17 March 1988       Volume 6 : Issue 31
@@ -65,7 +65,11 @@ struct vis_identifier {
 # ifndef CSRG_BASED
 #  include <sun/fbio.h>
 # else
-#  include <machine/fbio.h>
+#  ifdef __NetBSD__
+#   include <dev/sun/fbio.h>
+#  else
+#   include <machine/fbio.h>
+#  endif
 # endif
 #endif
 
