@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Cursor.c,v 3.41 2005/01/26 05:31:48 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Cursor.c,v 3.43 2005/10/14 15:16:32 tsi Exp $ */
 /*
  * Copyright (c) 1994-2005 by The XFree86 Project, Inc.
  * All rights reserved.
@@ -48,8 +48,8 @@
 
 
 #define NEED_EVENTS
-#include "X.h"
-#include "Xmd.h"
+#include <X11/X.h>
+#include <X11/Xmd.h>
 #include "input.h"
 #include "cursor.h"
 #include "mipointer.h"
@@ -63,7 +63,7 @@
 #include "xf86_OSproc.h"
 
 #ifdef XINPUT
-#include "XIproto.h"
+#include <X11/extensions/XIproto.h>
 #include "xf86Xinput.h"
 #endif
 
@@ -163,6 +163,11 @@ xf86SetViewport(ScreenPtr pScreen, int x, int y)
 {
   ScrnInfoPtr   pScr = XF86SCRNINFO(pScreen);
 
+#ifdef XFreeXDGA
+  if (DGAActive(pScr->scrnIndex))
+    return;
+#endif
+
   (*pScr->PointerMoved)(pScreen->myNum, x, y);
 }
 
