@@ -1,4 +1,4 @@
-/* $Xorg: utils.c,v 1.5 2001/02/09 02:05:24 xorgcvs Exp $ */
+/* $XFree86: xc/programs/Xserver/os/utils.c,v 3.116 2008/01/01 02:02:35 tsi Exp $ */
 /*
 
 Copyright 1987, 1998  The Open Group
@@ -49,7 +49,52 @@ OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
 OR PERFORMANCE OF THIS SOFTWARE.
 
 */
-/* $XFree86: xc/programs/Xserver/os/utils.c,v 3.95 2003/10/01 18:36:38 alanh Exp $ */
+/*
+ * Copyright (c) 1996-2006 by The XFree86 Project, Inc.
+ * All rights reserved.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject
+ * to the following conditions:
+ *
+ *   1.  Redistributions of source code must retain the above copyright
+ *       notice, this list of conditions, and the following disclaimer.
+ *
+ *   2.  Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer
+ *       in the documentation and/or other materials provided with the
+ *       distribution, and in the same place and form as other copyright,
+ *       license and disclaimer information.
+ *
+ *   3.  The end-user documentation included with the redistribution,
+ *       if any, must include the following acknowledgment: "This product
+ *       includes software developed by The XFree86 Project, Inc
+ *       (http://www.xfree86.org/) and its contributors", in the same
+ *       place and form as other third-party acknowledgments.  Alternately,
+ *       this acknowledgment may appear in the software itself, in the
+ *       same form and location as other such third-party acknowledgments.
+ *
+ *   4.  Except as contained in this notice, the name of The XFree86
+ *       Project, Inc shall not be used in advertising or otherwise to
+ *       promote the sale, use or other dealings in this Software without
+ *       prior written authorization from The XFree86 Project, Inc.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE XFREE86 PROJECT, INC OR ITS CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
 
 #ifdef __CYGWIN__
 #include <stdlib.h>
@@ -59,10 +104,10 @@ OR PERFORMANCE OF THIS SOFTWARE.
 #if defined(WIN32) && !defined(__CYGWIN__)
 #include <X11/Xwinsock.h>
 #endif
-#include "Xos.h"
+#include <X11/Xos.h>
 #include <stdio.h>
 #include "misc.h"
-#include "X.h"
+#include <X11/X.h>
 #include <X11/Xtrans.h>
 #include "input.h"
 #include "dixfont.h"
@@ -109,11 +154,16 @@ OR PERFORMANCE OF THIS SOFTWARE.
 #endif
 
 #ifdef XKB
-#include "XKBsrv.h"
+#include <X11/extensions/XKBsrv.h>
 #endif
+
 #ifdef XCSECURITY
 #define _SECURITY_SERVER
-#include "security.h"
+#include <X11/extensions/security.h>
+#endif
+
+#ifdef XPRINT
+#include "DiPrint.h"
 #endif
 
 #ifdef RENDER
@@ -145,10 +195,8 @@ int SyncOn  = 0;
 extern int SelectWaitTime;
 #endif
 
-#ifdef DEBUG
-#ifndef SPECIAL_MALLOC
-#define MEMBUG
-#endif
+#ifdef SPECIAL_MALLOC
+#undef MEMBUG
 #endif
 
 #if defined(SVR4) || defined(__linux__) || defined(CSRG_BASED)
@@ -165,9 +213,9 @@ long Memory_fail = 0;
 int userdefinedfontpath = 0;
 #endif /* sgi */
 
-char *dev_tty_from_init = NULL;		/* since we need to parse it anyway */
+const char *dev_tty_from_init = NULL;	/* since we need to parse it anyway */
 
-extern int dispatchExceptionAtReset;
+extern char dispatchExceptionAtReset;
 
 OsSigHandlerPtr
 OsSignal(sig, handler)
@@ -244,6 +292,7 @@ LockServer(void)
   int lfd, i, haslock, l_pid, t;
   char *tmppath = NULL;
   int len;
+  char port[20];
 
   if (nolock) return;
   /*
@@ -258,13 +307,14 @@ LockServer(void)
     FatalError("No TMP dir found\n");
 #endif
 
+  sprintf(port, "%d", atoi(display));
   len = strlen(LOCK_PREFIX) > strlen(LOCK_TMP_PREFIX) ? strlen(LOCK_PREFIX) :
 						strlen(LOCK_TMP_PREFIX);
-  len += strlen(tmppath) + strlen(display) + strlen(LOCK_SUFFIX) + 1;
+  len += strlen(tmppath) + strlen(port) + strlen(LOCK_SUFFIX) + 1;
   if (len > sizeof(LockFile))
-    FatalError("Display name `%s' is too long\n", display);
-  (void)sprintf(tmp, "%s" LOCK_TMP_PREFIX "%s" LOCK_SUFFIX, tmppath, display);
-  (void)sprintf(LockFile, "%s" LOCK_PREFIX "%s" LOCK_SUFFIX, tmppath, display);
+    FatalError("Display name `%s' is too long\n", port);
+  (void)sprintf(tmp, "%s" LOCK_TMP_PREFIX "%s" LOCK_SUFFIX, tmppath, port);
+  (void)sprintf(LockFile, "%s" LOCK_PREFIX "%s" LOCK_SUFFIX, tmppath, port);
 
   /*
    * Create a temporary file containing our PID.  Attempt three times
@@ -359,7 +409,7 @@ LockServer(void)
          */
         unlink(tmp);
 	FatalError("Server is already active for display %s\n%s %s\n%s\n",
-		   display, "\tIf this server is no longer running, remove",
+		   port, "\tIf this server is no longer running, remove",
 		   LockFile, "\tand start again.");
       }
     }
@@ -426,14 +476,57 @@ GiveUp(int sig)
     errno = olderrno;
 }
 
+/*ARGSUSED*/
+SIGVAL
+AbortServer(int sig)
+{
+    static Bool beenHere = FALSE;
+
+#if defined(SYSV) && defined(X_NOT_POSIX)
+    if (sig)
+	OsSignal(sig, SIG_IGN);
+#endif
+    /*
+     * Once entering here, ignore SIGINT and SIGTERM so that we don't
+     * re-enter.
+     */
+    OsSignal(SIGINT, SIG_IGN);
+    OsSignal(SIGTERM, SIG_IGN);
+    if (!beenHere) {
+	beenHere = TRUE;
+	if (sig == SIGINT)
+	    ErrorF("X server terminated by an interrupt signal.\n");
+	OsCleanup(TRUE);
+	AbortDDX();
+    } else {
+	ErrorF("Re-entering AbortServer.  Aborting without further cleanup.\n");
+    }
+    fflush(stderr);
+    if (CoreDump)
+	abort();
+    exit (1);
+}
+
 #ifndef DDXTIME
 CARD32
 GetTimeInMillis(void)
 {
     struct timeval  tp;
+    CARD32 val;
+    INT32 diff;
+    static CARD32 oldval = 0;
+    static CARD32 time = 0;
 
     X_GETTIMEOFDAY(&tp);
-    return(tp.tv_sec * 1000) + (tp.tv_usec / 1000);
+    val = (tp.tv_sec * 1000) + (tp.tv_usec / 1000);
+    if (oldval) {
+	diff = val - oldval;
+	if (diff > 0)
+	    time += diff;
+    }
+    oldval = val;
+
+    return time;
 }
 #endif
 
@@ -578,14 +671,16 @@ VerifyDisplayName(const char *d)
  * argc or any of the strings pointed to by argv.
  */
 void
-ProcessCommandLine(int argc, char *argv[])
+ProcessCommandLine(const int argc, const char *argv[])
 {
     int i, skip;
 
     defaultKeyboardControl.autoRepeat = TRUE;
 
-#ifdef PART_NET
-	PartialNetwork = TRUE;
+#ifdef NO_PART_NET
+    PartialNetwork = FALSE;
+#else
+    PartialNetwork = TRUE;
 #endif
 
     for ( i = 1; i < argc; i++ )
@@ -1182,24 +1277,24 @@ set_font_authorizations(char **authorizations, int *authlen, pointer client)
 void * 
 Xalloc(unsigned long amount)
 {
-    register pointer  ptr;
-	
-    if ((long)amount <= 0) {
-	return (unsigned long *)NULL;
-    }
+    pointer ptr;
+
     /* aligned extra on long word boundary */
     amount = (amount + (sizeof(long) - 1)) & ~(sizeof(long) - 1);
+
+    if ((long)amount <= 0)
+	return NULL;
 #ifdef MEMBUG
     if (!Must_have_memory && Memory_fail &&
 	((random() % MEM_FAIL_SCALE) < Memory_fail))
-	return (unsigned long *)NULL;
+	return NULL;
 #endif
-    if ((ptr = (pointer)malloc(amount))) {
-	return (unsigned long *)ptr;
+    if ((ptr = malloc(amount))) {
+	return ptr;
     }
     if (Must_have_memory)
 	FatalError("Out of memory");
-    return (unsigned long *)NULL;
+    return NULL;
 }
 
 /*****************
@@ -1210,20 +1305,22 @@ Xalloc(unsigned long amount)
 void *
 XNFalloc(unsigned long amount)
 {
-    register pointer ptr;
+    pointer ptr;
+
+    if (amount == 0)
+	return NULL;
 
-    if ((long)amount <= 0)
-    {
-        return (unsigned long *)NULL;
-    }
     /* aligned extra on long word boundary */
     amount = (amount + (sizeof(long) - 1)) & ~(sizeof(long) - 1);
-    ptr = (pointer)malloc(amount);
+
+    if ((long)amount <= 0)
+        FatalError("Bad request for memory");
+
+    ptr = malloc(amount);
     if (!ptr)
-    {
         FatalError("Out of memory");
-    }
-    return ((unsigned long *)ptr);
+
+    return ptr;
 }
 
 /*****************
@@ -1233,11 +1330,11 @@ XNFalloc(unsigned long amount)
 void *
 Xcalloc(unsigned long amount)
 {
-    unsigned long   *ret;
+    pointer ret;
 
-    ret = Xalloc (amount);
+    ret = Xalloc(amount);
     if (ret)
-	bzero ((char *) ret, (int) amount);
+	bzero (ret, (int) amount);
     return ret;
 }
 
@@ -1248,13 +1345,17 @@ Xcalloc(unsigned long amount)
 void *
 XNFcalloc(unsigned long amount)
 {
-    unsigned long   *ret;
+    pointer ret;
 
-    ret = Xalloc (amount);
-    if (ret)
-	bzero ((char *) ret, (int) amount);
-    else if ((long)amount > 0)
+    if (amount == 0)
+	return NULL;
+
+    ret = Xalloc(amount);
+    if (!ret)
         FatalError("Out of memory");
+
+    bzero (ret, (int) amount);
+
     return ret;
 }
 
@@ -1268,24 +1369,26 @@ Xrealloc(pointer ptr, unsigned long amount)
 #ifdef MEMBUG
     if (!Must_have_memory && Memory_fail &&
 	((random() % MEM_FAIL_SCALE) < Memory_fail))
-	return (unsigned long *)NULL;
+	return NULL;
 #endif
     if ((long)amount <= 0)
     {
 	if (ptr && !amount)
 	    free(ptr);
-	return (unsigned long *)NULL;
+	return NULL;
     }
     amount = (amount + (sizeof(long) - 1)) & ~(sizeof(long) - 1);
+    if ((long)amount <= 0)
+	return NULL;
     if (ptr)
-        ptr = (pointer)realloc((char *)ptr, amount);
+        ptr = realloc(ptr, amount);
     else
-	ptr = (pointer)malloc(amount);
+	ptr = malloc(amount);
     if (ptr)
-        return (unsigned long *)ptr;
+        return ptr;
     if (Must_have_memory)
 	FatalError("Out of memory");
-    return (unsigned long *)NULL;
+    return NULL;
 }
                     
 /*****************
@@ -1296,12 +1399,12 @@ Xrealloc(pointer ptr, unsigned long amount)
 void *
 XNFrealloc(pointer ptr, unsigned long amount)
 {
-    if (( ptr = (pointer)Xrealloc( ptr, amount ) ) == NULL)
+    if ((ptr = Xrealloc(ptr, amount)) == NULL)
     {
-	if ((long)amount > 0)
+	if (amount != 0)
             FatalError( "Out of memory" );
     }
-    return ((unsigned long *)ptr);
+    return ptr;
 }
 
 /*****************
@@ -1331,6 +1434,31 @@ OsInitAllocator (void)
 }
 #endif /* !INTERNAL_MALLOC */
 
+#if !defined(WORD64) && !defined(LONG64)
+
+void *
+Xllalloc(unsigned long long amount)
+{
+    if (amount & ~((unsigned long long)(unsigned long)(-1L))) return NULL;
+    return Xalloc(amount);
+}
+
+void *
+Xllrealloc(void *ptr, unsigned long long amount)
+{
+    if (amount & ~((unsigned long long)(unsigned long)(-1L))) return NULL;
+    return Xrealloc(ptr, amount);
+}
+
+void *
+Xllcalloc(unsigned long long amount)
+{
+    if (amount & ~((unsigned long long)(unsigned long)(-1L))) return NULL;
+    return Xcalloc(amount);
+}
+
+
+#endif
 
 char *
 Xstrdup(const char *s)
@@ -1360,6 +1488,52 @@ XNFstrdup(const char *s)
     return sd;
 }
 
+int
+Xasprintf(char **ret, const char *format, ...)
+{
+    char *s;
+    va_list args;
+    int status;
+
+    if (!ret || !format)
+	return -1;
+
+#ifdef HAS_ASPRINTF
+    va_start(args, format);
+    status = vasprintf(&s, format, args);
+    va_end(args);
+    if (status != -1 && s) {
+	*ret = Xstrdup(s);
+	free(s);
+	if (!*ret)
+	    status = -1;
+    } else
+	*ret = NULL;
+    return status;
+#else
+#define TMP_SIZE 4000
+    s = xcalloc(1, TMP_SIZE);
+    if (!s) {
+	*ret = NULL;
+	return -1;
+    }
+    va_start(args, format);
+    status = vsnprintf(s, TMP_SIZE, format, args);
+    va_end(args);
+    if (status > TMP_SIZE - 1)
+	status = TMP_SIZE - 1;
+    if (status < TMP_SIZE - 1) {
+	*ret = xrealloc(s, status + 1);
+	if (!*ret) {
+	    xfree(s);
+	    status = -1;
+	}
+    } else
+	*ret = s;
+    return status;
+#endif
+}
+
 #ifdef SMART_SCHEDULE
 
 unsigned long	SmartScheduleIdleCount;
@@ -1402,8 +1576,9 @@ SmartScheduleStartTimer (void)
     timer.it_value.tv_sec = 0;
     timer.it_value.tv_usec = SmartScheduleInterval * 1000;
     return setitimer (ITIMER_REAL, &timer, 0) >= 0;
-#endif
+#else
     return FALSE;
+#endif
 }
 
 #ifdef SMART_SCHEDULE_POSSIBLE
@@ -1519,7 +1694,7 @@ OsReleaseSignals (void)
  */
 
 int
-System(char *command)
+System(const char *command)
 {
     int pid, p;
 #ifdef SIGCHLD
@@ -1567,7 +1742,7 @@ static struct pid {
 } *pidlist;
 
 pointer
-Popen(char *command, char *type)
+Popen(const char *command, const char *type)
 {
     struct pid *cur;
     FILE *iop;
@@ -1641,7 +1816,7 @@ Popen(char *command, char *type)
 
 /* fopen that drops privileges */
 pointer
-Fopen(char *file, char *type)
+Fopen(const char *file, const char *type)
 {
     FILE *iop;
 #ifndef HAS_SAVED_IDS_AND_SETEUID
@@ -1711,24 +1886,26 @@ Fopen(char *file, char *type)
     ErrorF("Popen: `%s', fp = %p\n", command, iop);
 #endif
 
-    return iop;
 #else
     int ruid, euid;
 
     ruid = getuid();
     euid = geteuid();
     
-    if (seteuid(ruid) == -1) {
-	    return NULL;
-    }
+    if (seteuid(ruid) == -1)
+	return NULL;
+
     iop = fopen(file, type);
 
     if (seteuid(euid) == -1) {
+	if (iop)
 	    fclose(iop);
-	    return NULL;
+	return NULL;
     }
-    return iop;
+
 #endif /* HAS_SAVED_IDS_AND_SETEUID */
+
+    return iop;
 }
 
 int
@@ -1854,11 +2031,11 @@ enum BadCode {
       "the \"super user\" (root).\n"
 
 void
-CheckUserParameters(int argc, char **argv, char **envp)
+CheckUserParameters(const int argc, const char **argv, char **envp)
 {
     enum BadCode bad = NotBad;
     int i = 0, j;
-    char *a, *e = NULL;
+    const char *a, *e = NULL;
 #if defined(__QNX__) && !defined(__QNXNTO__)
     char cmd_name[64];
 #endif
@@ -1995,9 +2172,146 @@ CheckUserParameters(int argc, char **argv, char **envp)
  */
 
 #ifdef USE_PAM
-#include <security/pam_appl.h>
-#include <security/pam_misc.h>
 #include <pwd.h>
+#include <security/pam_appl.h>
+#ifndef sun
+#ifdef _OPENPAM
+#include <security/openpam.h>
+#undef   misc_conv
+#define  misc_conv openpam_ttyconv
+#else  /* _OPENPAM */
+#include <security/pam_misc.h>
+#endif /* _OPENPAM */
+#else  /* sun */
+
+#include <termio.h>
+
+static volatile int SIGINTed;
+
+static void
+SIGINThandler(int signo)
+{
+    SIGINTed = 1;
+}
+
+/* A conversation function for Solaris */
+static int
+misc_conv(int nummsgs, struct pam_message **pamms, struct pam_response **pamrs,
+	  void *data)
+{
+    struct pam_message *pamm;
+    struct pam_response *pamr;
+    struct termio tty;
+    void (*handler_save)(int);
+    int echo, c;
+    unsigned short flags_save = 0;
+    char input[PAM_MAX_RESP_SIZE + 1], *pchar;
+
+    if ((nummsgs <= 0) || (nummsgs >= PAM_MAX_NUM_MSG)) {
+	ErrorF("PAM authentication error, invalid number of messages:"
+		"  %d, (max %d)\n", nummsgs, PAM_MAX_NUM_MSG);
+	return PAM_CONV_ERR;
+    }
+
+    pamr = xcalloc(nummsgs, sizeof(struct pam_response));
+    if (!pamr) {
+	ErrorF("PAM authentication error, memory allocation failure\n");
+	return PAM_CONV_ERR;
+    }
+
+    *pamrs = pamr;
+    for (pamm = *pamms;  nummsgs > 0;  nummsgs--, pamm++, pamr++) {
+	if (pamm->msg == NULL) {
+	    ErrorF("PAM authentication error, NULL message\n");
+	    break;
+	}
+
+	/* Strip any trailing newline */
+	pchar = pamm->msg + strlen(pamm->msg);
+	if (*pchar == '\n')
+	    *pchar = '\0';
+
+	echo = 0;
+	switch (pamm->msg_style) {
+	default:
+	    ErrorF("PAM authentication error, unknown message type:  %d\n",
+		   pamm->msg_style);
+	    goto fail;
+
+	case PAM_ERROR_MSG:
+	    (void) fputs(pamm->msg, stderr);
+	    (void) fputc('\n', stderr);
+	    break;
+
+	case PAM_TEXT_INFO:
+	    (void) fputs(pamm->msg, stdout);
+	    (void) fputc('\n', stdout);
+	    break;
+
+	case PAM_PROMPT_ECHO_ON:
+	    echo = 1;
+	    /* Fall through */
+
+	case PAM_PROMPT_ECHO_OFF:
+	    (void) fputs(pamm->msg, stderr);
+
+	    SIGINTed = 0;
+	    handler_save = signal(SIGINT, SIGINThandler);
+
+	    if (!echo) {
+		(void) ioctl(fileno(stdin), TCGETA, &tty);
+		flags_save = tty.c_lflag;
+		tty.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL);
+		(void) ioctl(fileno(stdin), TCSETAF, &tty);
+	    }
+
+	    flockfile(stdin);
+
+	    pchar = input;
+	    while (!SIGINTed &&
+		 ((c = getchar_unlocked()) != '\n') &&
+		 (c != '\r') &&
+		 (c != EOF)) {
+		if (pchar < (input + PAM_MAX_RESP_SIZE))
+		    *pchar++ = c;
+	    }
+	    *pchar = '\0';
+
+	    if (!SIGINTed)
+		pamr->resp = xstrdup(input);
+	    bzero(input, sizeof(input));
+
+	    funlockfile(stdin);
+
+	    if (!echo) {
+		tty.c_lflag = flags_save;
+		(void) ioctl(fileno(stdin), TCSETAW, &tty);
+		(void) fputc('\n', stdout);
+	    }
+
+	    (void) signal(SIGINT, handler_save);
+
+	    if (SIGINTed || !pamr->resp)
+		goto fail;
+	    break;
+	}
+    }
+
+    return PAM_SUCCESS;
+
+fail:
+    for (;  pamr >= *pamrs;  pamr--) {
+	if (pamr->resp) {
+	    bzero(pamr->resp, strlen(pamr->resp));
+	    xfree(pamr->resp);
+	}
+    }
+    xfree(*pamrs);
+    *pamrs = NULL;
+    return PAM_CONV_ERR;
+}
+
+#endif /* sun */
 #endif /* USE_PAM */
 
 void
@@ -2016,7 +2330,8 @@ CheckUserAuthorization(void)
     if (getuid() != geteuid()) {
 	pw = getpwuid(getuid());
 	if (pw == NULL)
-	    FatalError("getpwuid() failed for uid %d\n", getuid());
+	    FatalError("getpwuid() failed for uid %ld\n",
+			(unsigned long)getuid());
 
 	retval = pam_start("xserver", pw->pw_name, &conv, &pamh);
 	if (retval != PAM_SUCCESS)
@@ -2042,3 +2357,24 @@ CheckUserAuthorization(void)
     }
 #endif
 }
+
+int
+getArgc()
+{
+    return argcGlobal;
+}
+
+const char **
+getArgvp()
+{
+    return argvGlobal;
+}
+
+const char *
+getArgv(int i)
+{
+    if (i >= 0 && i < argcGlobal)
+	return argvGlobal[i];
+    else
+	return NULL;
+}
